package dpexamples.day2;

import java.io.File;

public class FileSystem {

    public static void main(String[] args) {
    	
    	// AbstractFile root = buildDemoFilesystem();
    	AbstractFile root = populate(new File("."));

    	CountFilesVisitor fileCounter = new CountFilesVisitor();
    	traverse(root, fileCounter);
		System.out.println("we have in total " + fileCounter.counter + " files.");

        SummarizeFileSizesVisitor sfsv = new SummarizeFileSizesVisitor();
        traverse(root, sfsv);
        System.out.println("these files take " + sfsv.totalSize + " bytes of harddisk space.");

		// will sys.out.println itself
    	traverse(root, new FindEmptyDirectoriesVisitor());
    }

    private static AbstractFile populate(File file) {
        AbstractFile abstractFile;
        if (file.isDirectory()) {
            abstractFile = new Directory(file.getName());
            for (File child : file.listFiles()) {
                abstractFile.add(populate(child));
            }
        } else {
            abstractFile = new RegularFile(file.getName());
        }
        abstractFile.size = file.length();
        return abstractFile;
	}

    static void traverse(AbstractFile file, FileVisitor v) {
    	file.accept(v);
    	for (AbstractFile child : file.contains) {
    		traverse(child, v);
    	}    	
    }

	private static Root buildDemoFilesystem() {
		Root root = new Root();
    	Drive c = new Drive("C:");
    	root.add(c);
    	Directory windows = new Directory("windows");
    	Directory sys = new Directory("system32");
    	Directory temp = new Directory("temp");
    	c.add(windows);
    	c.add(temp);
    	c.add(new Directory("temp2"));
    	windows.add(sys);
    	windows.add(new RegularFile("notepad.exe"));
    	sys.add(new RegularFile("hardwaredriver.sys"));
    	temp.add(new RegularFile("editorcontent.tmp"));
    	Drive d = new Drive("D:");
    	d.add(new RegularFile("somemusic.mp3"));
    	d.add(new RegularFile("setup.exe"));
    	root.add(d);
		return root;
	}
    
}

class Drive extends Directory {
	Drive(String name) {
		super(name);
	}
}
class Root extends AbstractFile {
	Root() {
		super("My Computer");
	}	
}

